# frozen_string_literal: true

# Provides the data access layer.
# If +use_redis_cache+ is enabled, calls to the data access layer will go through Redis first.
# If +use_redis_cache+ is disabled, calls will directly go to DynamoDB.
# In both of these cases, the contract is the same.
Application.boot(:data_access) do |app|
  init do
    require 'aws_account_status/data_access/dynamodb_data_access_proxy'
    require 'aws_account_status/data_access/redis_data_access'
    require 'aws_account_status/data_access/account_api_status_sql_data_access'
  end

  start do
    # Get dependencies into +app+.
    use :dynamodb # Provides a connection factory.
    use :cache
    use :logger

    table_name = target[:settings].dynamodb_account_region_status_table_name

    # Register the normal +data_access+ component to abstract away the two data stores.
    # If we're using the caching layer, this registers a Redis wrapper. If not, it passes straight through
    # to the DynamoDB client.
    register(:data_access, call: false) do
      use_caching = target[:settings].use_redis_cache == 1

      # Use a proxy to avoid the expensive initialization each time.
      dynamo_data_access = ::AwsAccountStatus::DataAccess::DynamoDbDataAccessProxy.new(table_name)

      if use_caching
        app['logger'].debug { 'USE_REDIS_CACHE active. Using Redis caching layer over DynamoDB.' }
        ::AwsAccountStatus::DataAccess::Redis.new(app['cache.redis'], dynamo_data_access, app['logger'])
      else
        app['logger'].debug { 'USE_REDIS_CACHE inactive. Skipping Redis cache. Directly accessing DynamoDB.' }
        dynamo_data_access
      end
    end

    # Register spend monitor table with dynamoDB client.
    register(:spend_data_access, call: false) do
      spend_table_name = target[:settings].dynamodb_region_spend_status_table_name
      app['logger'].debug { "spend_data_access using table name: #{spend_table_name}" }

      ::AwsAccountStatus::SpendDataAccess::DynamoDb.new(spend_table_name, Application.resolve('dynamodb.client'))
    end
    
    register(:api_status_data_access, call: false) do
      ::AwsAccountStatus::DataAccess::AccountApiStatusSQLDataAccess.new
    end
  end
end
