# frozen_string_literal: true

# Provide common settings for the service.
#
# All of these settings can be accessed via `Application[:settings]`,
# or using IoC injection, as:
#
# @example dry-auto_inject
#   class Foo
#     include Import[:settings]
#     def call
#       puts "Running in pod #{settings.pod_name}"
#     end
#   end
#
# Each of these settings can be provided as an environment variable;
# or in a `.env` file; or in a `.env.development` file, where the
# value of `$RACK_ENV` is used as the suffix.  Each of the settings
# becomes an attribute on the settings object.
Application.boot(:settings, from: :system) do
  init do
    require 'cht-lib-fabric/dry_types'
  end

  settings do
    # Booleans are represented as ints because there's no readily-accessible boolean type.

    key :use_creds_from_instance_metadata,
        ::Platform::DryTypes::Coercible::Integer.default(0)

    key :use_redis_cache,
        ::Platform::DryTypes::Coercible::Integer.default(0)

    key :redis_host,
        ::Platform::DryTypes::Coercible::String.default('localhost')
    key :redis_port,
        ::Platform::DryTypes::Coercible::Integer.default(6379)
    # Boolean as int. Enable logging Redis commands.
    key :redis_debug,
        ::Platform::DryTypes::Coercible::Integer.default(0)

    # Wiring to reach the token-provider JWKS endpoint and validate tokens
    key :jwks_uri,
        ::Platform::DryTypes::Coercible::String.default('')
    key :linkerd_dtab,
        ::Platform::DryTypes::Coercible::String.default('')
    key :pod_name,
        ::Platform::DryTypes::Coercible::String.default('__pod__')
    # If non-empty, allow the dev RSA key
    key :developer_jwt_signing,
        ::Platform::DryTypes::Coercible::String.default('')

    # Wiring to reach Rollbar
    key :rollbar_token,
        ::Platform::DryTypes::Coercible::String.default('')
    key :rollbar_env,
        ::Platform::DryTypes::Coercible::String.default('')
    key :k8s_pod,
        ::Platform::DryTypes::Coercible::String.default('')

    # DynamoDB connection settings
    key :dynamodb_account_region_status_table_name,
        ::Platform::DryTypes::Coercible::String.default('cht-account-region-status')
    key :dynamodb_url,
        ::Platform::DryTypes::Coercible::String.default(nil)
    key :dynamodb_region,
        ::Platform::DryTypes::Coercible::String.default(ENV['AWS_REGION'] || 'us-east-1')

    # DynamoDB Spend-Monitor connection settings
    key :dynamodb_region_spend_status_table_name,
        ::Platform::DryTypes::Coercible::String.default('cht-region-spend-status')
    key :spend_dynamodb_endpoint,
        ::Platform::DryTypes::Coercible::String.default(nil)
    key :spend_dynamodb_region,
        ::Platform::DryTypes::Coercible::String.default(ENV['AWS_REGION'] || 'us-east-1')

    # Wavefront integration
    key :wf_host,
        ::Platform::DryTypes::Coercible::String.default('')
    key :wf_port,
        ::Platform::DryTypes::Coercible::Integer.default(2181)
    key :k8s_ns,
        ::Platform::DryTypes::Coercible::String.default('aws-region-status')

    # Statsd integration
    key :statsd_host, ::Platform::DryTypes::Coercible::String.default(ENV['STATSD_HOST'] || 'localhost')
    key :statsd_port, ::Platform::DryTypes::Coercible::Integer.default(ENV['STATSD_PORT'] || 8125)

    # Kafka
    key :kafka_host,
        ::Platform::DryTypes::Coercible::String.default(ENV['KAFKA_HOST'] || 'localhost:9092')
    key :kafka_topic_prefix,
        ::Platform::DryTypes::Coercible::String.default('')

    # MySQL
    key :mysql_config_file,
        ::Platform::DryTypes::Coercible::String.default('database.yml.erb')

    # Integration AWS
    key :integration_aws_uri,
        ::Platform::DryTypes::Coercible::String.default(ENV['AWS_URI'] || 'localhost:8870')

    # Org Management
    key :org_management_uri,
        ::Platform::DryTypes::Coercible::String.default(ENV['ORG_MANAGEMENT_URI'] || 'http://localhost:8860')

    # Authorization
    key :authorization_uri,
      ::Platform::DryTypes::Coercible::String.default(ENV['AUTHORIZATION_URI'] || 'http://localhost:8850')
  end
end
